# ScanPrint Quick Start Guide

This is a guide to help people start developing software for the ScanPrint SP5001 USB scanner.

## Prerequisites
This guide assumes that you have experience developing in C/C++ on Windows and already have Visual Studio installed. The sample code has been built and tested on Windows 10 and Visual Studio 2019 or Visual Studion 2022 (the free community version includes everything you'll need). 

During installation of Visual Studio you will need to select the 'Desktop development with C++' workload. 

## Installing
All the software needed can be downloaded from the ScanPrint website at <http://scanprint.co.uk/Drivers>. The main files needed are:   

**ScanPrint-Setup.exe** is the Combined Auto Installer for the required Windows Drivers, which are needed before software can connect to the scanner and also for the Engbench2 test application, which allows you to test and configure the scanner.
  
**ScanPrint-Setup-X.Y.Z.zip** contains separate installers for the required Windows drivers and the Engbench2 test application. These are an **alternative** to the main auto installer above. The separate installers are:

1. scanprintdriver-XXbit.msi --- The 32 or 64 bit Windows driver for the Scanprint scanner / printer.
2. Scanprint.msi --- Installs the Engbench2 test and config application 
3. VC_redist.x86 --- The Visual C++ Redistributable Packages install run-time components that are required to run C++ applications built using Visual Studio.


**ScanPrint-Dev-Kit-X.Y.Z.zip** is the development kit and it contains the essential binaries plus SDK files and development example source files needed for development.
1. Examples --- simple C++ examples
2. PrinterDemo --- C# example 
3. ScanDemo --- C# example 
4. SDK --- The API release DLL and header and lib files needed by applications
5. Twain32 API ( Deprecated ) 
6. scanprintdriver-XXbit.msi - the 32 or 64 bit Windows driver for the Scanprint scanner and Scanprint printer.
7. EngBench2 - binaries for the test and config app

NOTE - In many cases, depending on the test facilities you build into your application you may not need EngBench2 in which case you will only need to distribute the **CyberSc.dll** (the Scanner API), **Cfg1xxx.dat** (the Scanner config) files and obviously the scanprintdriver-XXbit.msi device drivers.

## Getting Started

The most useful part of the guide is the sample code in \\examples\\. These are  simple command line C++ programs that will scan an image from the scanner and save it as a bitmap. 

## Sync or Async API

There are two APIs you can use talk to the scanner. Blocking synchronous function calls or a callback based asynchronous calls. 

### Sync API

A very simple program to read an image from the scanner is below. 

```
#include <windows.h>
#include "CyberSC.h"
int main(){
	ScSyncInit();
	BITMAPINFO* pBitmap = ScSyncScan(false);	
	//Do something with the bitmap.
}
```
A fuller version with error checking, and freeing the bit map is in the examples\\sync-cpp\\

`ScSyncInit` Initialise the scanner for the synchronous API.
`ScSyncScan` Block waiting for an image to be scanned or an error. If the parameter is true the function treats debug messages as errors and will return on every debug message. 

### Async API

A sample showing the async api is in examples\\async-cpp\\

The scanner initialization function is `ScInit`. This takes two functions to be used as callbacks. One for when an image has been scanned and one as an error reporting method, e.g.  `ScInit((FARPROC)ImageScanned, (FARPROC)ErrorNotify)`. The callback functions will be called on a different thread.

Then to tell the scanner to be ready to scan call `ScSCalibrate` with `CV_CAL_STARTSCAN`, e.g. `ScSCalibrate(CV_CAL_STARTSCAN, 0)`

## Building and running the sample.

### From Visual Studio.

Open "examples/examples.sln"
Rebuild the projects from the menu.

### From the command line

Open the "x86 Native Tools Command Prompt for VS 2019 or VS2022" 

```
cd examples
msbuild
```

### Running

The command line applications can be run from the inside Visual Studio under the debugger or from the command line in the examples/Debug directory. Make sure that the Cfg1xxx.dat file is in the working directory. They  will write save the scanned image to the working directory. They will terminate when an image has been scanned, an error has been received or if Ctrl-C is pressed.
