﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ScanDemo
{
    public partial class MainWindow : Form
    {
        private CyberCs.ScanDataCallback m_fullImage;
        private CyberCs.ScanPartDataCallback m_partImage;
        private CyberCs.ErrorReportCallback m_errorReport;

        private Queue<Image> m_images;

        private CpuEater m_cputEater = new CpuEater();
        

        public MainWindow()
        {
            InitializeComponent();
            m_fullImage = ScanData;
            m_partImage = ScanPartData;
            m_errorReport = ErrorMessage;
            m_images = new Queue<Image>();
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            var ret = CyberCs.ScInit(m_fullImage, m_errorReport);
            if (ret == 0)
            {
                MessageBox.Show("Unable to init scanner");
                return;
            }

            paperStatusTm.Enabled = true; 

        }


        private void StartScnBtn_Click(object sender, EventArgs e)
        {
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_STOPSCAN, 0);
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_SET_RESO, (int)Resolutions.EACRmFULL2);
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_SET_DEV, (int)DeviceCode.EDEV200A4_1728);
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_SET_LIGHT, 160); //set light 184
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_STARTSCAN, 0);
            CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_FORCESCAN, 0);

        }

        private void ReciveImage(Bitmap image)
        {
            if(scanedImage.Image == null)
            {
                scanedImage.Image = image;
            } 
            else
            {
                m_images.Enqueue(image);
            }

            nextImageBtn.Enabled = m_images.Count > 0;
            
        }

        private void nextImageBtn_Click(object sender, EventArgs e)
        {
            scanedImage.Image = m_images.Dequeue();
            nextImageBtn.Enabled = m_images.Count > 0;
        }


        private void partScanBtn_Click(object sender, EventArgs e)
        {
            int[] steps = { 100 , 100, 100};
            int numSteps = steps.Length;
            CyberCs.ScSetupPartScan(steps, numSteps, m_partImage);

        }

        private void rejectBtn_Click(object sender, EventArgs e)
        {
            CyberCs.ScRejectPaper();
        }

        private void loadBtn_Click(object sender, EventArgs e)
        {
            if(m_cputEater.IsRunning)
            {
                m_cputEater.Stop();
            }
            else
            {
                m_cputEater.Run();
            }
        }


        private void paperStatusTm_Tick(object sender, EventArgs e)
        {
            var paper = CyberCs.ScSCalibrate(ConfigSettings.CV_CAL_PAPER, 0);
            if(paper != 0)
            {
                paperLbl.Text = "Paper Detected";
            }
            else
            {
                paperLbl.Text = "No Paper";
            }

        }

        // From here down these are callback functions. These are not in the UI thread.

        private void ScanData(ref CyberImg img)
        {
            var bitmap = new Bitmap(img.wWid, img.wHei, img.wWidBy, PixelFormat.Format1bppIndexed, img.pIm);
            Invoke((Action)delegate() { ReciveImage(new Bitmap(bitmap)); });
        }

        private bool ScanPartData(ref CyberImg img)
        {
            var bitmap = new Bitmap(img.wWid, img.wHei, img.wWidBy, PixelFormat.Format1bppIndexed, img.pIm);
            Invoke((Action)delegate () { ReciveImage(new Bitmap(bitmap)); });
            return true;
            //return false;
        }

        private void ErrorMessage(int error, int type)
        {

        }

    }
}
